/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.diff.api.sample;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;

import ro.sync.diff.api.DiffContentTypes;
import ro.sync.diff.api.DiffException;
import ro.sync.diff.api.DiffOptions;
import ro.sync.diff.api.DiffPerformerFactory;
import ro.sync.diff.api.DiffProgressEvent;
import ro.sync.diff.api.DiffProgressListener;
import ro.sync.diff.api.Difference;
import ro.sync.diff.api.DifferencePerformer;

/**
 * Sample application demonstrating the use of the {@link DifferencePerformer}.
 */
public class DiffXMLFilesSample {
  /**
   * Main method.
   * 
   * @param args Should be two arguments specifying the XML files to compare.
   * 
   * @throws DiffException If the diff cannot be performed.
   * @throws IOException If the license or the files to compare couldn't be found, or any other IO problem occurs.
   */
  public static void main(String[] args) throws DiffException, IOException {
    if (args == null || args.length < 2) {
      System.out.println("Please specify the two XML files to be compared!");
    } else {
      //Register license.
      DiffPerformerFactory.registerLicenseKey(new FileReader(new File("test/licenseKeyDiffServer.txt")));
      
      // Create the diff performer.
      final DifferencePerformer differencePerformer = DiffPerformerFactory.createDiffPerformer();
      
      
      // Thread used to stop the diff process when 'c' is entered in the console.
      Thread stopThread = new Thread(new Runnable() {
        public void run() {
          System.out.println("Press 'C' to cancel the diff process!");
          try {
            int read = System.in.read();
            if ('c' == read || 'C' == read) {
              differencePerformer.stop();
            }
          } catch (IOException e) {
            e.printStackTrace();
          }
        }
      });
      
      // Create the diff options. 
      DiffOptions diffOptions = new DiffOptions();
      diffOptions.setMaxNumberOfDiffs(3000);
      diffOptions.setIgnoreWhitespaces(true);
      diffOptions.setAlgorithm(DiffOptions.XML_FAST);
      System.out.println("Comparing '" + args[0] + "' with '" + args[1] + "' using XML fast algorithm!");
      
      // Create the diff progress listener.
      DiffProgressListener diffProgressListener = new DiffProgressListener() {
        public void start() {
          System.out.println("Diff started!");
        }
        public void update(DiffProgressEvent progressEvent) {
          System.out.println("Progress "  + progressEvent.getCount() + "%");
        }
        public void finished() {
          System.out.println("Diff finished!");
        }
      };
      
      try {
        // Get the files to be compared. 
        File file1 = new File(args[0]);
        File file2 = new File(args[1]);
        
        
        // Start the thread used to cancell the diff process.
        stopThread.start();
        
        // Perform diff.
        List<Difference> differences = differencePerformer.performDiff(
            new FileReader(file1), 
            new FileReader(file2), 
            file1.toURI().toURL().toString(), 
            file2.toURI().toURL().toString(), 
            DiffContentTypes.XML_CONTENT_TYPE, 
            diffOptions, 
            diffProgressListener);

        // Show the differences in console.
        System.out.println("-------------------------------------");
        System.out.println("The differences are:\n" + differences);
      } finally {
        stopThread.interrupt();
        System.exit(0);
      }
    }
  }
}