/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.diff.api;

/**
 * Holds options needed to decide which diff algorithm and which diff options
 * will be used.
 */
public class DiffOptions {
  /**
   * The algorithm is automatically chosen based on content type, text length, options etc.
   */
  public static final int AUTO = 0;

  /**
   * Algorithm based on characters comparison.
   */
  public static final int CHARACTERS_COMPARE = 1;

  /**
   * Algorithm based on words comparison.
   */
  public static final int WORDS_COMPARE = 2;

  /**
   * Algorithm based on line by line comparison.
   */
  public static final int LINES_COMPARE = 3;

  /**
   * Algorithm based on comparing the syntax tokens.
   */
  public static final int SYNTAX_AWARE = 4;

  /**
   * Fast XML hybrid heuristic algorithm. 
   */
  public static final int XML_FAST = 5;

  /**
   * XML accurate diff based on Sasha and Zhang algorithm.
   */
  public static final int XML_ACCURATE = 6;
  
  /**
   * The algorithm to use for comparison.
   */
  private int algorithm;

  /**
   * The maximum number of modifications allowed by the diff algorithm.
   * This option allows you to specify the maximum number of differences between your documents 
   * that you might be interested to see (using the Longest Common Subsequence Algorithm).
   */
  private int maxNumberOfDiffs;

  /**
   * <code>true</code> if whitespaces should be ignored.
   */
  private boolean ignoreWhitespaces;
  
  /**
   * <code>true</code> if processing instructions should be ignored.
   * Applies when using XML aware algorithms.
   */
  private boolean ignorePI;

  /**
   * <code>true</code> if comments should be ignored.
   */
  private boolean ignoreComments;

  /**
   * <code>true</code> if CDATA sections should be ignored.
   * Applies when using XML aware algorithms.
   */
  private boolean ignoreCDATA;

  /**
   * <code>true</code> if DOCTYPE section should be ignored.
   * Applies when using XML aware algorithms.
   */
  private boolean ignoreDOCTYPE;
  
  /**
   * <code>true</code> if text sections should be ignored.
   * Applies when using XML aware algorithms.
   */
  private boolean ignoreText;

  /**
   * <code>true</code> if attributes order should be ignored.
   * Applies when using XML aware algorithms.
   */
  private boolean ignoreAttributesOrder;
  
  /**
   * <code>true</code> if prefixes are ignored and comparing by namespace only.
   * Applies when using XML aware algorithms.
   */
  private boolean ignorePrefixes;
  
  /**
   * <code>true</code> if namespaces are ignored when comparing.
   * Applies when using XML aware algorithms.
   */
  private boolean ignoreNamespaces;
  
  /**
   * <code>true</code> if namespace declarations are to be ignored ignored.
   * Applies when using XML aware algorithms.
   */
  private boolean ignoreNamespaceDeclarations;
  
  /**
   * <code>true</code> if entities content is ignored when comparing.
   * Applies when using XML aware algorithms.
   */
  private boolean ignoreEntities;
  
  /**
   * <code>true</code> if sequences representing empty elements like 
   * <br/><code>&lt;a/> and &lt;a>&lt;/a></code> 
   * <br/>will be considered identical.
   * Applies when using XML aware algorithms.
   */
  private boolean ignoreStateOfEmptyElements;

  /**
   * If <code>true</code>, it considers adjacent differences as one.
   * If <code>false</code>, every difference is presented separately.
   * Applies when using XML aware algorithms.
   */
  private boolean mergeAdjacentDifferences;
  
  /**
   * <code>true</code> if end tags for the modified elements that have the same name will
   * be marked as different.
   * Applies when using XML aware algorithms.
   */
  private boolean markEndTagsAsDifferent;

  /**
   * Constructor.
   * 
   * @param algorithm The used algorithm, one of the constants:
   * <ul>
   * <li>{@link #AUTO}</li>
   * <li>{@link #CHARACTERS_COMPARE}</li>
   * <li>{@link #LINES_COMPARE}</li>
   * <li>{@link #SYNTAX_AWARE}</li>
   * <li>{@link #WORDS_COMPARE}</li>
   * <li>{@link #XML_ACCURATE}</li>
   * <li>{@link #XML_FAST}</li>
   * </ul>
   * @param ignoreWhitespaces <code>true</code> if differences consisting only in whitespaces should be ignored. 
   * @param maxNumberOfDiffs The maximum number of modifications you are interested in. If the document contain
   *        a larger number of differences the diff will only signal that they are different and stop after reaching the limit.
   *        
   * @param ignorePI <code>true</code> if processing instructions should be ignored. 
   *        It applies when using XML aware algorithms.
   * @param ignoreComments <code>true</code> if comments should be ignored. 
   *        It applies when using XML aware algorithms.
   * @param ignoreCDATA <code>true</code> if CDATA sections should be ignored. 
   *        It applies when using XML aware algorithms.
   * @param ignoreDOCTYPE <code>true</code> if DOCTYPE sections should be ignored. 
   *        It applies when using XML aware algorithms.
   * @param ignoreText <code>true</code> if differences in the text content should be ignored.
   *        It applies when using XML aware algorithms.
   * @param ignoreAttributesOrder <code>true</code> if the order of the attributes should be ignored.
   *        It applies when using XML aware algorithms.
   * @param ignorePrefixes <code>true</code> if the prefixes should be ignored. 
   *        Even if this is set the comparing takes into account the namespacea.
   *        It applies when using XML aware algorithms.
   * @param ignoreNamespaces <code>true</code> if namespaces should be ignored.
   *        It applies when using XML aware algorithms.
   * @param ignoreNamespaceDeclarations <code>true</code> if the namespace declaration ("xmlns" attributes) should be ignored.
   *        It applies when using XML aware algorithms.
   * @param ignoreEntities <code>true</code> if entity content should be ignored.
   *        It applies when using XML aware algorithms.
   * @param mergeAdjacentDifferences <code>true</code> if adjacent differences should be merged and present as a single difference.
   *        Applies when using XML aware algorithms.
   * @param markEndTagsAsDifferent <code>true</code> if end tags with the same name for elements that are different in other sense
   *        will be shown as different also.
   *        It applies when using XML aware algorithms.
   * @param ignoreStateOfEmptyElements <code>true</code> if sequences representing empty elements like 
   *        <br/><code>&lt;a/> and &lt;a>&lt;/a></code> 
   *        <br/>will be considered identical.
   *        It applies when using XML aware algorithms.
   */
  public DiffOptions(int algorithm, boolean ignoreWhitespaces, int maxNumberOfDiffs, boolean ignorePI,
      boolean ignoreComments, boolean ignoreCDATA, boolean ignoreDOCTYPE,
      boolean ignoreText, boolean ignoreAttributesOrder, boolean ignorePrefixes, 
      boolean ignoreNamespaces, boolean ignoreNamespaceDeclarations, boolean ignoreEntities,
      boolean mergeAdjacentDifferences, boolean markEndTagsAsDifferent, boolean ignoreStateOfEmptyElements) {
    this.algorithm = algorithm;
    this.ignorePI = ignorePI;
    this.ignoreComments = ignoreComments;
    this.ignoreCDATA = ignoreCDATA;
    this.ignoreWhitespaces = ignoreWhitespaces;
    this.ignoreDOCTYPE = ignoreDOCTYPE;
    this.ignoreText = ignoreText;
    this.ignoreAttributesOrder = ignoreAttributesOrder;
    this.ignorePrefixes = ignorePrefixes;    
    this.ignoreNamespaces = ignoreNamespaces;
    this.ignoreNamespaceDeclarations = ignoreNamespaceDeclarations;
    this.ignoreEntities = ignoreEntities;
    this.mergeAdjacentDifferences = mergeAdjacentDifferences;
    this.markEndTagsAsDifferent = markEndTagsAsDifferent;
    this.ignoreStateOfEmptyElements = ignoreStateOfEmptyElements;
    this.maxNumberOfDiffs = maxNumberOfDiffs;
  }
  
  /**
   * Default constructor for default options.
   */
  public DiffOptions() {
    this(DiffOptions.AUTO, 
        false, 
        1000,
        false, 
        false, 
        false, 
        false,
        false, 
        false, 
        false, 
        false, 
        false, 
        true, 
        false, 
        false, 
        false);
  }
  

  /**
   * @return Returns The used algorithm, one of the constants:
   * <ul>
   * <li>{@link #AUTO}</li>
   * <li>{@link #CHARACTERS_COMPARE}</li>
   * <li>{@link #LINES_COMPARE}</li>
   * <li>{@link #SYNTAX_AWARE}</li>
   * <li>{@link #WORDS_COMPARE}</li>
   * <li>{@link #XML_ACCURATE}</li>
   * <li>{@link #XML_FAST}</li>
   * </ul>
   */
  public int getAlgorithm() {
    return algorithm;
  }

  /**
   * @param algorithm The algorithm to use, one of the constants:
   * <ul>
   * <li>{@link #AUTO}</li>
   * <li>{@link #CHARACTERS_COMPARE}</li>
   * <li>{@link #LINES_COMPARE}</li>
   * <li>{@link #SYNTAX_AWARE}</li>
   * <li>{@link #WORDS_COMPARE}</li>
   * <li>{@link #XML_ACCURATE}</li>
   * <li>{@link #XML_FAST}</li>
   * </ul>
   */
  public void setAlgorithm(int algorithm) {
    this.algorithm = algorithm;
  }

  /**
   * @return Returns The maximum number of modifications you are interested in. If the document contain
   *        a larger number of differences the diff will only signal that they are different and stop after reaching the limit.
   */
  public int getMaxNumberOfDiffs() {
    return maxNumberOfDiffs;
  }

  /**
   * @param maxNumberOfDiffs The maximum number of modifications you are interested in. If the document contain
   *        a larger number of differences the diff will only signal that they are different and stop after reaching the limit.
   */
  public void setMaxNumberOfDiffs(int maxNumberOfDiffs) {
    this.maxNumberOfDiffs = maxNumberOfDiffs;
  }
  
  /**
   * @return Returns <code>true</code> if differences consisting only in whitespaces will be ignored. 
   */
  public boolean isIgnoreWhitespaces() {
    return ignoreWhitespaces;
  }

  /**
   * @param ignoreWhitespaces <code>true</code> if differences consisting only in whitespaces should be ignored. 
   */
  public void setIgnoreWhitespaces(boolean ignoreWhitespaces) {
    this.ignoreWhitespaces = ignoreWhitespaces;
  }

  /**
   * @return Returns <code>true</code> if processing instructions are ignored. 
  *        It applies when using XML aware algorithms.
   */
  public boolean isIgnorePI() {
    return ignorePI;
  }

  /**
   * @param ignorePI <code>true</code> if processing instructions should be ignored. 
  *        It applies when using XML aware algorithms.
   */
  public void setIgnorePI(boolean ignorePI) {
    this.ignorePI = ignorePI;
  }

  /**
   * @return Returns <code>true</code> if comments are be ignored. 
  *        It applies when using XML aware algorithms.
   */
  public boolean isIgnoreComments() {
    return ignoreComments;
  }

  /**
   * @param ignoreComments <code>true</code> if comments should be ignored. 
  *        It applies when using XML aware algorithms.
   */
  public void setIgnoreComments(boolean ignoreComments) {
    this.ignoreComments = ignoreComments;
  }

  /**
   * @return Returns <code>true</code> if CDATA sections are ignored. 
  *        It applies when using XML aware algorithms.
   */
  public boolean isIgnoreCDATA() {
    return ignoreCDATA;
  }

  /**
   * @param ignoreCDATA <code>true</code> if CDATA sections should be ignored. 
  *        It applies when using XML aware algorithms.
   */
  public void setIgnoreCDATA(boolean ignoreCDATA) {
    this.ignoreCDATA = ignoreCDATA;
  }

  /**
   * @return Returns <code>true</code> if DOCTYPE sections are ignored. 
  *        It applies when using XML aware algorithms.
   */
  public boolean isIgnoreDOCTYPE() {
    return ignoreDOCTYPE;
  }

  /**
   * @param ignoreDOCTYPE <code>true</code> if DOCTYPE sections should be ignored. 
  *        It applies when using XML aware algorithms.
   */
  public void setIgnoreDOCTYPE(boolean ignoreDOCTYPE) {
    this.ignoreDOCTYPE = ignoreDOCTYPE;
  }

  /**
   * @return Returns code>true</code> if differences in the text content is ignored.
   *        It applies when using XML aware algorithms.
   */
  public boolean isIgnoreText() {
    return ignoreText;
  }

  /**
   * @param ignoreText code>true</code> if differences in the text content should be ignored.
   *        It applies when using XML aware algorithms.
   */
  public void setIgnoreText(boolean ignoreText) {
    this.ignoreText = ignoreText;
  }

  /**
   * @return Returns <code>true</code> if the order of the attributes is ignored.
   *        It applies when using XML aware algorithms.
   */
  public boolean isIgnoreAttributesOrder() {
    return ignoreAttributesOrder;
  }

  /**
   * @param ignoreAttributesOrder <code>true</code> if the order of the attributes should be ignored.
   *        It applies when using XML aware algorithms.
   */
  public void setIgnoreAttributesOrder(boolean ignoreAttributesOrder) {
    this.ignoreAttributesOrder = ignoreAttributesOrder;
  }

  /**
   * @return Returns code>true</code> if the prefixes are ignored. 
   *        Even if this is set the comparing takes into account the namespaces.
   *        It applies when using XML aware algorithms.
   */
  public boolean isIgnorePrefixes() {
    return ignorePrefixes;
  }

  /**
   * @param ignorePrefixes code>true</code> if the prefixes should be ignored. 
   *        Even if this is set the comparing takes into account the namespaces.
   *        It applies when using XML aware algorithms.
   */
  public void setIgnorePrefixes(boolean ignorePrefixes) {
    this.ignorePrefixes = ignorePrefixes;
  }
  
  /**
   * @return Returns <code>true</code> if namespaces are ignored.
   *        It applies when using XML aware algorithms.
   */
  public boolean isIgnoreNamespaces() {
    return ignoreNamespaces;
  }

  /**
   * @param ignoreNamespaces <code>true</code> if namespaces should be ignored.
   *        It applies when using XML aware algorithms.
   */
  public void setIgnoreNamespaces(boolean ignoreNamespaces) {
    this.ignoreNamespaces = ignoreNamespaces;
  }

  /**
   * @return Returns <code>true</code> if the namespace declarations ("xmlns" attributes) are ignored.
   *        It applies when using XML aware algorithms.
   */
  public boolean isIgnoreNamespaceDeclarations() {
    return ignoreNamespaceDeclarations;
  }

  /**
   * @param ignoreNamespaceDeclarations <code>true</code> if the namespace declarations ("xmlns" attributes) should be ignored.
   *        It applies when using XML aware algorithms.
   */
  public void setIgnoreNamespaceDeclarations(boolean ignoreNamespaceDeclarations) {
    this.ignoreNamespaceDeclarations = ignoreNamespaceDeclarations;
  }

  /**
   * @return Returns <code>true</code> if entity content is ignored.
   *        It applies when using XML aware algorithms.
   */
  public boolean isIgnoreEntities() {
    return ignoreEntities;
  }

  /**
   * @param ignoreEntities <code>true</code> if entity content should be ignored.
   *        It applies when using XML aware algorithms.
   */
  public void setIgnoreEntities(boolean ignoreEntities) {
    this.ignoreEntities = ignoreEntities;
  }

  /**
   * @return Returns <code>true</code> if sequences representing empty elements like 
   *        <br/><code>&lt;a/> and &lt;a>&lt;/a></code> 
   *        <br/>are considered identical.
   *        It applies when using XML aware algorithms.
   */
  public boolean isIgnoreStateOfEmptyElements() {
    return ignoreStateOfEmptyElements;
  }

  /**
   * @param ignoreStateOfEmptyElements <code>true</code> if sequences representing empty elements like 
   *        <br/><code>&lt;a/> and &lt;a>&lt;/a></code> 
   *        <br/>will be considered identical.
   *        It applies when using XML aware algorithms.
   */
  public void setIgnoreStateOfEmptyElements(boolean ignoreStateOfEmptyElements) {
    this.ignoreStateOfEmptyElements = ignoreStateOfEmptyElements;
  }

  /**
   * @return Returns <code>true</code> if adjacent differences are merged and present as a single difference.
   *         It applies when using XML aware algorithms.
   */
  public boolean isMergeAdjacentDifferences() {
    return mergeAdjacentDifferences;
  }

  /**
   * @param mergeAdjacentDifferences <code>true</code> if adjacent differences should be merged and present as a single difference.
   *        It applies when using XML aware algorithms.
   */
  public void setMergeAdjacentDifferences(boolean mergeAdjacentDifferences) {
    this.mergeAdjacentDifferences = mergeAdjacentDifferences;
  }

  /**
   * @return Returns <code>true</code> if end tags with the same name for elements that are 
   *        different in other sense are shown as different also.
   *        It applies when using XML aware algorithms.
   */
  public boolean isMarkEndTagsAsDifferent() {
    return markEndTagsAsDifferent;
  }

  /**
   * @param markEndTagsAsDifferent <code>true</code> if end tags with the same name for elements 
   *        that are different in other sense will be shown as different also.
   *        It applies when using XML aware algorithms.
   */
  public void setMarkEndTagsAsDifferent(boolean markEndTagsAsDifferent) {
    this.markEndTagsAsDifferent = markEndTagsAsDifferent;
  }
}