/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.diff.api;

import java.io.Reader;
import java.util.List;

/**
 * The {@link DifferencePerformer} is used to compare two resources of a given content type using
 * a set of options. The result of the diff is a list with the differences between the resources. 
 */
public interface DifferencePerformer {
  /**
   * Performs the diff operation between the resources represented by the two readers.
   * The resources are compared taking into account the content type and the provided options.
   * <br>Please note that the content of the readers will be processed before comparison
   * and all end lines will be replaced with <code>\n</code>.  
   * 
   * @param leftContentReader The first resource content reader.
   *        It cannot be <code>null</code>.
   * @param rightContentReader The second resource content reader.
   *        It cannot be <code>null</code>.
   * @param leftSystemId The left resource URL as string.
   *        It can be <code>null</code> when a system ID is not relevant.
   * @param rightSystemId The right resource URL as string.
   *        It can be <code>null</code> when a system ID is not relevant.
   * @param contentType The content type of the two resources, constants from {@link DiffContentTypes}. 
   *        It is used only when the diff algorithm takes into account the
   *        tokens of the language associated with the resources.
   *        It can be <code>null</code> when the used algorithm is not syntax aware.
   * @param diffOptions The options used by the diff performer to perform the comparison.
   *        It can be <code>null</code> meaning a default set of options wil be used.
   * @param diffProgressListener The {@link DiffProgressListener} notified about the progress of the diff.
   *        It can be <code>null</code> when the diff progress doesn't need to be monitored.
   * @return The list with the differences. The differences are represented by {@link Difference} objects.
   *        If the resources are identical or the diff fails the list will be empty.
   * @throws DiffException If the diff operation fails or it is stopped before it finishes.
   */
  List<Difference> performDiff(
      Reader leftContentReader, 
      Reader rightContentReader,
      String leftSystemId,
      String rightSystemId,
      String contentType, 
      DiffOptions diffOptions, 
      DiffProgressListener diffProgressListener) throws DiffException;
  
  /**
   * Signal to the diff performer that it must stop.
   */
  void stop();
}